@media all {
    .btn-icon-aasrp {
        height: 2rem;
        margin: 0.1rem;
        width: 2rem;
    }

    .btn-icon-aasrp-status:focus {
        border: inherit;
    }

    .srp-payout-amount-changed {
        font-weight: bold;
    }

    .srp-link-actions,
    .srp-request-actions {
        white-space: nowrap;
    }

    .srp-dashboard-user-isk-cost,
    .srp-dashboard-total-isk-cost {
        font-weight: bold;
    }

    .card-srp-request-overview-cell {
        margin-bottom: 1rem;
    }

    .card-srp-request-overview-cell:last-child {
        margin-bottom: 0;
    }

    .modal-srp-details {
        margin-bottom: 2rem;
    }

    .aasrp-character-portrait,
    .aasrp-evetype-icon {
        margin-right: 0.5rem;
    }

    .aasrp-character-portrait-character-name small {
        font-size: 80%;
    }

    .aa-srp .text-aasrp-form-description {
        font-weight: 400;
    }

    .aa-srp-modal-header > h5 {
        border-bottom: 1px solid var(--bs-secondary-bg-subtle);
    }

    .srp-request-payout-amount-editable .srp-payout-amount {
        border-bottom: dashed 1px rgb(0 136 204);
    }

    .srp-request-status-rejected .srp-request-payout-amount-editable .srp-payout-amount {
        border-bottom: dashed 1px rgb(192 192 192);
    }

    .srp-request-payout-amount-editable .srp-payout-amount-changed {
        border-bottom: dashed 1px rgb(204 71 0) !important;
    }

    .modal-srp-details-request-history .request-history-comment:last-child {
        border-bottom: none;
        padding-bottom: initial;
    }

    .srp-request-bulk-actions-checkbox .checkbox {
        padding: 0.25rem 0.5rem;
    }

    .srp-request-bulk-actions-checkbox .checkbox label {
        font-size: 1.25rem;
    }

    .card-srp-request-bulk-actions {
        right: 1.7rem;
        top: 5.5rem;
        z-index: 1;
    }

    .card-srp-request-bulk-actions .card-header .fa-solid {
        transition: 0.25s transform ease-in-out;
    }

    .card-srp-request-bulk-actions .card-header.collapsed .fa-solid {
        transform: rotate(-90deg);
    }
}

/* Editable Popup
------------------------------------------------------------------------------------- */
@media all {
    .editable-popup .arrow {
        display: none;
    }
}
