Your SRP request regarding your {{ ship|safe }} lost during {{ srp_name|safe }} has been {{ status }}.

__**Request Details:**__
**SRP Code:** {{ srp_code }}
**Request Code:** {{ request_code }}
**Reviser:** {{ reviser|safe }}
{% if comment %}
__**Comment:**__
{{ comment|safe }}
{% endif %}
If you have any questions regarding your SRP request, feel free to contact your request reviser. Please make sure to always add the SRP code, and the request code with your inquiry.
